<?php
/**
 * Protect your templates
 *
 * @category Prestashop
 * @category Module
 * @author Samdha <contact@samdha.net>
 * @copyright Samdha
 * @license commercial license see license.txt
 * @version 1.0.1.0
 * @license logo http://www.gnu.org/copyleft/gpl.html GPL
 * @see logo http://icones.pro/diabolique-visage-image-png.html
**/

class Samdha_EmptyCache_Main extends Samdha_Commons_Module
{
    public $short_name = 'empcache';
    private $cache_dirs = array();
    public $id_addons = 0;

    public function __construct()
    {
        $this->name = 'emptycache';
        if (version_compare(_PS_VERSION_, '1.4.0.0', '<')) {
            $this->tab = 'Tools';
        } else {
            $this->tab = 'administration';
        }
        $this->version = '1.2.0';

        parent::__construct();

        /* The parent construct is required for translations */
        $this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('Empty cache');
        $this->description = $this->l('Allows you to clean Prestashop cache');
        if (version_compare(_PS_VERSION_, '1.5.0.0', '<')) {
            $this->cache_dirs = array(
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'smarty'.DIRECTORY_SEPARATOR.'compile',
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'smarty'.DIRECTORY_SEPARATOR.'cache',
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'smarty_v2'.DIRECTORY_SEPARATOR.'cache',
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'smarty_v2'.DIRECTORY_SEPARATOR.'compile',
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'cache',
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'img'.DIRECTORY_SEPARATOR.'tmp'
            );
        } else {
            $this->cache_dirs = array(
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'smarty'.DIRECTORY_SEPARATOR.'compile',
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'smarty'.DIRECTORY_SEPARATOR.'cache',
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'smarty_v2'.DIRECTORY_SEPARATOR.'cache',
                _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'tools'.DIRECTORY_SEPARATOR.'smarty_v2'.DIRECTORY_SEPARATOR.'compile',
                defined('_PS_CACHE_DIR_')?_PS_CACHE_DIR_:(_PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'cache'),
                defined('_PS_TMP_IMG_DIR_')?_PS_TMP_IMG_DIR_:(_PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'img'.DIRECTORY_SEPARATOR.'tmp')
            );
        }
    }

    public function install()
    {
        if (version_compare(_PS_VERSION_, '1.5.0.0', '<')) {
            return (
                parent::install()
                && $this->registerHook('header')
            );
        } else {
            return (
                parent::install()
                && $this->registerHook('header')
                && $this->registerHook('displayBackOfficeHeader')
            );
        }
    }

    public function displayForm($token)
    {
        $tabs = array();
        $tabs[] = array('href' => '#tabInformation', 'display_name' => $this->l('Information', 'main'));

        $this->smarty->assign(array(
            'tabs'            => $tabs,
            'cache_size'         => $this->humanReadableFilesize($this->getCacheSize()),
            'displayed'         => $this->getFrontCookie()->{$this->name} == $this->config->token,
        ));
        // Display Form
        return parent::displayForm($token);
    }

    public function postProcess($token)
    {
        if (!$this->config->token) {
            $this->config->token = Tools::passwdGen();
        }
        if (Tools::isSubmit('cleanCache')) {
            $this->emptyTheCache();
            Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&conf=1&token='.$token);
        }
        if (Tools::isSubmit('showIcon')) {
            $this->getFrontCookie()->{$this->name} = $this->config->token;
            Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&conf=6&token='.$token);
        }
        if (Tools::isSubmit('hideIcon')) {
            unset($this->getFrontCookie()->{$this->name});
            Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&conf=6&token='.$token);
        }
        return parent::postProcess($token);
    }

    // set default config
    public function getDefaultConfig()
    {
        $result = array(
            'token' => null, // password for cookie
        );

        return $result;
    }

    public function hookdisplayBackOfficeHeader($params)
    {
        return $this->hookHeader($params);
    }

    public function hookHeader($params)
    {
        if (version_compare(_PS_VERSION_, '1.5.0.0', '>=')) {
            $context = Context::getContext();
            $smarty = $context->smarty;
            $link = $context->link;
        } else {
            global $smarty, $link;
        }
        if (isset($this->config->token)
            && ($this->getFrontCookie()->{$this->name} == $this->config->token)
        ) {
            if (Tools::getValue($this->name)) {
                $this->emptyTheCache();
                Tools::redirectAdmin($this->getUrlWithoutParameter());
            }
            $smarty->assign(array(
                'cacheSize' => $this->humanReadableFilesize($this->getCacheSize()),
                'this_path' => $this->_path
            ));

            $filename = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.$this->name.'.php';
            $hook_path = version_compare(_PS_VERSION_, '1.5.0.0', '<')?'views/templates/hook/':'';
            return $this->display($filename, $hook_path.$this->name.'.tpl');
        }
    }

    private function emptyTheCache()
    {
        foreach ($this->cache_dirs as $dir) {
            if (file_exists($dir)) {
                $this->emptydir($dir);
            }
        }
    }

    private function getCacheSize()
    {
        $size = 0;
        foreach ($this->cache_dirs as $dir) {
            $size += $this->dirsize($dir);
        }
        return $size;
    }

    /** Usefull functions **/

    /**
     * return the current url without a parameter
     * @see Link::getUrlWith
     *
     * @param string $key the parameter
     * @return string
     **/
    public function getUrlWithoutParameter()
    {
        $url = isset($_SERVER['REQUEST_URI'])?$_SERVER['REQUEST_URI']:($_SERVER['SCRIPT_NAME'].'?'.$_SERVER['QUERY_STRING']);
        $url = preg_replace('/'.$this->name.'=1&?/', '', $url);
        return rtrim($url, '?&');
    }

    /**
     * Returns a human readable filesize
     *
     * @author      wesman20 (php.net)
     * @author      Jonas John
     * @version     0.3
     * @link        http://www.jonasjohn.de/snippets/php/readable-filesize.htm
     */
    public function humanReadableFilesize($size)
    {

        // Adapted from: http://www.php.net/manual/en/function.filesize.php

        $mod = 1024;

        $units = explode(' ', 'B KB MB GB TB PB');
        for ($i = 0; $size > $mod; $i++) {
            $size /= $mod;
        }

        return round($size, 2) . ' ' . $units[$i];
    }

    // http://www.php.net/manual/fr/function.rmdir.php#98622
    private function emptydir($dir)
    {
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != "." && $object != ".." && $object != 'index.php') {
                    if (is_dir($dir.DIRECTORY_SEPARATOR.$object)) {
                        $this->emptydir($dir.DIRECTORY_SEPARATOR.$object);
                    } else {
                        @unlink($dir.DIRECTORY_SEPARATOR.$object);
                    }
                }
            }
            reset($objects);
        }
    }

    private function dirsize($dir)
    {
        $size = 0;
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != "." && $object != ".." && $object != 'index.php') {
                    if (is_dir($dir.DIRECTORY_SEPARATOR.$object)) {
                        $size += $this->dirsize($dir.DIRECTORY_SEPARATOR.$object);
                    } else {
                        $size += filesize($dir.DIRECTORY_SEPARATOR.$object);
                    }
                }
            }
            reset($objects);
        }
        return $size;
    }

    private function getFrontCookie()
    {
        if (version_compare(_PS_VERSION_, '1.5.0.0', '>=')) {
            if (defined('_PS_ADMIN_DIR_')) {
                $cookie = Context::getContext()->cookie;
            } else {
                $cookie = new Cookie('psAdmin');
            }
        } else {
            if (defined('_PS_ADMIN_DIR_')) {
                $cookie = new Cookie('ps');
            } else {
                global $cookie;
            }
        }
        return $cookie;
    }
}
